;
; 	GUI - ported from MMBASIC on the Micromite, code originally by user CaptainBoing on
;	http://www.fruitoftheshed.com/MMBasic.A-Simple-GUI-pack.ashx
;
;	coded and compiled in Prog8 version 9.4 for the Commander X16
;	S. Yockey - September 2023
;
%import gfx2
%import floats
%import conv
%import syslib
%import string

main {

	;for mouse functions
	uword @requirezp mx 
	uword @requirezp my 
	
	;bool ProcessTouch 
	ubyte btn
	byte P				;next available object

	
	
	const ubyte Objs = 50	;max number of GUI objects (must add string storage space in 'object' if you increase this)
	const ubyte CGY = 26
	const ubyte CGH = 29

;objects	
	; button 'keycaps'
	const ubyte ZER = 20
	const ubyte ONE = 21
	const ubyte TWO = 22
	const ubyte THR	= 23
	const ubyte FOR = 24
	const ubyte FIV = 25
	const ubyte SIX = 26
	const ubyte SEV = 27
	const ubyte ATE = 28
	const ubyte NIN = 29
	const ubyte DOT = 30		;decimal point
	const ubyte EQU = 31		;calculate
	const ubyte ADD = 32		;ADD
	const ubyte SUB = 33		;subtract
	const ubyte MUL = 34		;multiply
	const ubyte DIV = 35		;divide
	const ubyte CLR = 36		;clear entry
	const ubyte SQT = 37		;square root
	const ubyte SQR = 38		;square
	const ubyte SIN	= 39		;sine
	const ubyte COS = 40		;cosine
	const ubyte TAN = 41		;tangent
	const ubyte POW = 42		;power = x^n
	const ubyte PIE = 43		;Pi constant
	const ubyte MMS = 44		;store in memory
	const ubyte MMR = 45		;memory recall
	const ubyte MMP = 46		;add to memory
	const ubyte EXP = 47		;exponentiation  = 1 times 10^n
	const ubyte MCL = 48		;memory clear
	const ubyte HIN = 49		;HEX text
	const ubyte BNN = 50		;BIN text
	const ubyte DEC = 54		;decimal mode
	const ubyte TRU = 55		;truncate



	
	const ubyte NEG = 70		;negate
	
	;textbox
	const ubyte RES = 90		;result decimal
	const ubyte HEX = 91		;result hexadecimal
	const ubyte BIN = 92		;result binary
	
	const ubyte IND = 95
	
	const ubyte HLP = 98		;info box
	const ubyte TTL = 99 		;title
	
;other constants
	const ubyte TERM1 = 0
	
	const ubyte TERM2 = 2
	
	const ubyte XPO1 = 3
	
	const ubyte XPO2 = 4
	
	const float mhex = 2.81474977e14
	
	ubyte OPER = 0
	
	;variables for term1, term2 and result
	float t1
	float t2
	float xp
	float result
	float mem
	byte place				;used to factor how to place additonal digits in input number, 1,2,3 left of decimal, -1,-2,-3 right of decimal
	bool decimal			;used to track whether the decimal point has been pressed
	bool z_flg = false				;used to denote result was zero
	
	;strings for term1, term2 and result
	str ts1 = "000000000000"
	str ts2 = "000000000000"
	str rs = "00000000000000000000"
	str dec = "."
	
	; for hex number conversion
	str answer = "????????????????"
	str num = iso:"0123456789ABCDEF"	
	
	str banswer = "????????????????"
	str bnum = iso:"01"
	str ca1 = "???"
	
	;variable to track what operation the calculation is at (term1 -> operation -> term2 -> result)
	ubyte status = TERM1
	;ubyte mode = DEC
	
	;+/- negate symbol and division symbol
	str NGS = iso:"\u00b1"
	str DVS = iso:"\u00f7"
	str PIC = iso:"\u0001"		;pi character
	str XPE = iso:"\u00ca"
	str QRS = iso:"\u0004"				;square symbol
	str PWW = iso:"\u0002"		;power symbol
	str RUT = iso:"\u0003"		;square root symbol

	
	;pi characters
	ubyte[] pc = [$00,$3e,$54,$14,$14,$14,$22,$00]
	;power character
	ubyte[] pw = [$05,$02,$95,$50,$20,$20,$40,$80]
	;square root character
	ubyte[] sr = [$00,$0f,$10,$10,$20,$a0,$40,$00]
;	;square root character
	ubyte[] sq = [$02,$05,$01,$8a,$55,$20,$50,$88]	
	

	
	sub start() {
	
		byte q
		byte f
		byte g
		byte h
		byte j
		ubyte m
		;uword t
		;uword i
		str tmp = "????????????????????"
		

		
		gfx2.screen_mode(4)
		gfx2.text_charset(1)
		
		;make pi character at charset location 1
		for m in 0 to 7 {
			cx16.vpoke(1,$f008+m,pc[m]) }
			
		for m in 0 to 7 {
			cx16.vpoke(1,$f010+m,pw[m]) }
		
		for m in 0 to 7 {
			cx16.vpoke(1,$f018+m,sr[m]) }
			
		for m in 0 to 7 {
			cx16.vpoke(1,$f020+m,sq[m]) }			
		
		gfx2.fillrect(0,0,320,240,CGH)
		
		init()
	
		cx16.mouse_config(1, 320/8, 240/8)
		
		void string.copy("",ts1)
		void string.copy("",ts2)
		
		
		repeat {
		
			if GUI.GetTouch() {									; if left button is pressed
				q = GUI.ProcessTouch()							; process it
				
				h = GUI.ObjFind(HLP)
				f = GUI.ObjFind(RES)
				g = GUI.ObjFind(IND)
				
				object.txdata[h] = " "
				GUI.ObjDraw(HLP)
				
				
				;get button press and update textbox
				when q {						
					CLR -> {								;for now, this clears everything
						decimal = false
						place = 0
						t1 = 0
						t2 = 0
						result = 0
						;void string.copy(" ",ts1) ; = ""
						;void string.copy(" ",ts2) ; = ""
						ts1[0] = 0
						ts2[0] = 0
						status = TERM1
						void string.copy("0.",object.txdata[f])
						GUI.ObjDraw(RES)						
												
						float_to_hex(t1)
						j = GUI.ObjFind(HEX)
						void string.copy(answer,object.txdata[j])
						GUI.ObjDraw(HEX)								
										
						float_to_bin(t1)
						j = GUI.ObjFind(BIN)
						void string.copy(banswer,object.txdata[j])
						GUI.ObjDraw(BIN)														
					}						
						
							
						
					;decimal key			
					DOT -> {							
						if status < XPO1 {				
						if decimal == false {		
							decimal = true		
							place = -1							

							if status == TERM1 {
								if result != 0 {
									ts1[0] = 0
									t1 = 0
									object.txdata[f] = ts1
									GUI.ObjDraw(RES)
								}
								cat(dec,ts1)												
								object.txdata[f] = ts1	}						
							else {						
								cat(dec,ts2)												
								object.txdata[f] = ts2	
							}																										
						}												
						}											
					}						
											
					;number keys
					20,21,22,23,24,25,26,27,28,29 -> {					
						conv.str_ub(q as ubyte-20)		
					
					;handle exponent portion of scientific notation number		
						if status >= XPO1 {
							goto n1 }
							

					;to start a completely new 1st term
					if status == TERM1 and t1 != 0 {
						if result != 0 or z_flg == true {
							ts1[0] = 0
							ts1[1] = 0
							ts1[2] = 0
							ts1[3] = 0
							t1 = 0 
							result = 0
							z_flg = false
						}
					}	

										
n1:					do_digit(conv.string_out,q as ubyte-20)		



					if status == TERM1 {
						do_hex(t1)
						do_bin(t1) }
					else {
						do_hex(t2)
						do_bin(t2) 
						
						}
						

												
					}		
						
					PIE -> {
						if status == TERM1 {
							ts1[0] = 0
							t1 = 3.141592654
							ts1 = "3.141592654"
							object.txdata[f] = ts1							
							do_hex(t1)						
							do_bin(t1)					
						}
						else {
							t2 = 3.141592654
							ts2 = "3.141592654"
							object.txdata[f] = ts2						
							do_hex(t2)					
							do_bin(t2)			
							}
					
						GUI.ObjDraw(RES)
						}
						
					EXP -> {				
						if status == TERM1 {
							ca1[0] = 69
							ca1[1] = 0
							cat(ca1,ts1) 
							status = XPO1
							place = 0

						}
						else {
							ca1[0] = 69
							ca1[1] = 0
							cat(ca1,ts2) 
							status = XPO2
							place = 0
						}
					}
							
					NEG -> {	
						if status >= XPO1 {
							if status == XPO1 {
								xp = -(xp)
								if xp < 0 {
									ins("-",ts1) }
								else {
									ins("+",ts1) }
								object.txdata[f] = ts1
								}
							else {
								xp = -(xp)
								if xp < 0 {
									ins("-",ts2) }
								else {
									ins("+",ts2) }
								object.txdata[f] = ts2	
								}						
														
						}							
						else {
						
						if status == TERM1 {	
							t1 = -(t1)	
							if t1 < 0 {	
								cat("-",ts1) 
							}
							else {
								cat("+",ts1)
							}
						
						object.txdata[f] = ts1
						do_hex(t1)
						do_bin(t1)
						}
						else {
							t2 = -(t2)	
							if t2 < 0 {	
								cat("-",ts2) 
							}
							else {
								cat("+",ts2)
							}
						
						object.txdata[f] = ts2						
						do_hex(t2)
						do_bin(t2)										
						}
					}
					}		
							
								
					ADD -> {										
						if status == XPO1 {																				
							xp = floats.pow(10,xp)																						
							t1 = t1 * xp																						
							}								
							
						status = TERM2
						decimal = false
						place = 0
						OPER = ADD
						object.txdata[f] = " "							
						void string.copy("+",object.txdata[g]) ; = "+"												
						GUI.ObjDraw(IND)					
					}		
							
					SUB -> {				
						if status == XPO1 {																				
							xp = floats.pow(10,xp)																						
							t1 = t1 * xp																						
							}											
						status = TERM2
						decimal = false
						place = 0
						OPER = SUB
						object.txdata[f] = " "							
						void string.copy("-",object.txdata[g]) ; = "-"												
						GUI.ObjDraw(IND)												
					}							
						
					MUL -> {																											
						if status == XPO1 {																				
							xp = floats.pow(10,xp)																						
							t1 = t1 * xp																						
							}																																
						status = TERM2
						decimal = false
						place = 0
						OPER = MUL
						object.txdata[f] = " "													
						void string.copy("x",object.txdata[g]) ; = "x"												
						GUI.ObjDraw(IND)																																																							
					}							
											
					SQR -> {						
						if status == XPO1 {																				
							xp = floats.pow(10,xp)																						
							t1 = t1 * xp																						
							}											
						%asm{{					;check that multiplication does not result in overflow				
							lda p8_t1				
							clc				
							adc p8_t1				
							bcc p8_s1				
							and #$80				
							bne p8_s2				
							}}														
s1:							t2 = t1																		
							status = TERM2								
							OPER = MUL						
							place = 0						
							decimal = false						
							goto e1						
s2:						
							result = t1
							void string.copy("error",rs)
							goto e2																											
					}						
								
					POW -> {			
						status = TERM2			
						place = 0			
						decimal = false			
						OPER = POW			
						object.txdata[f] = " "
					}			
					
					TRU -> {
						status = TERM2
						place = 0
						decimal = false
						OPER = TRU
						object.txdata[f] = " "
						goto e1
					}
					
					SIN -> {
;						if t1 == 0 {
;							goto e3 }
							
						if t1 == 180 {
							t2 = 0
							OPER = MUL
							place = 0
							decimal = false
							object.txdata[f] = " "
							goto e1 }
						else {
							status = TERM2
							place = 0
							decimal = false
							OPER = SIN
							object.txdata[f] = " "
							goto e1
						}
					}					
										
					COS -> {
						status = TERM2
						place = 0
						decimal = false
						OPER = COS
						object.txdata[f] = " "
						goto e1
					}										
													
					TAN -> {
						if t1 == 90 {
							result = t1
							void string.copy("error",rs)
							goto e2							
							}						
													
						if t1 == 270 {
							result = t1
							void string.copy("error",rs)
							goto e2							
							}													
							
						if t1 == 0 {
							goto e3 }
							
						if t1 == 180 {
							goto e3 }
						else {
							status = TERM2
							place = 0
							decimal = false
							OPER = TAN
							object.txdata[f] = " "
							goto e1
						}
					}													
								
								
					SQT -> {			
						if status == XPO1 {																				
							xp = floats.pow(10,xp)																						
							t1 = t1 * xp																						
							}							
						status = TERM2		
						OPER = SQT			
						place = 0	
						decimal = false	
						goto e1	
						}	
									
						
					DIV -> {	
						if status == XPO1 {																				
							xp = floats.pow(10,xp)																						
							t1 = t1 * xp																						
							}						
						status = TERM2	
						decimal = false	
						place = 0	
						OPER = DIV	
						object.txdata[f] = " "													
						void string.copy(DVS,object.txdata[g]) ; = DVS											
						GUI.ObjDraw(IND)							
					}						
					
					
					EQU -> {													
e1:						if status == XPO2 {																				
							xp = floats.pow(10,xp)																						
							t2 = t2 * xp																						
							}																														
																																										
						do_oper()
						
						
e2:						object.txdata[f] = rs		;new
						t1 = result
						
						if result == 0 {
							z_flg = true 
							result = 1 
							t1 = .00000000000001 }
						else {
							z_flg = false }
											
						do_hex(t1)
						do_bin(t1)

						void string.copy(rs,ts1)
						ts2[0] = 0
						t2 = 0
						xp = 0
						;result = 0
						status = TERM1
						g = GUI.ObjFind(IND)
						void string.copy(" ",object.txdata[g]) ; = " "												
						GUI.ObjDraw(IND)
					}
					
					MMS -> {
						if status == TERM1 {
							mem = t1 }
						else {
							mem = t2 }
					}
						
											
					MMR -> {				
						f = GUI.ObjFind(RES)
						if status == TERM1 {
							t1 = mem 
							floats.MOVFM(&t1)
							void floats.FOUT()			
							void string.copy($0100,object.txdata[f])											
							do_hex(t1)
							do_bin(t1)
							}
						else {
							
							t2 = mem 
							floats.MOVFM(&t2)
							void floats.FOUT()
							f = GUI.ObjFind(RES)
							void string.copy($0100,tmp)												
							object.txdata[f] = tmp											
							do_hex(t2)
							do_bin(t2)
						}
					}
					
					MMP -> {
						if status == TERM1 {
							if t1 != 0 {
								mem += t1 }
							else {
								if t2 != 0  {
									mem += t2 }
							}
						}	
					}
					
					MCL -> {
						mem = 0				
					}
												
												
					
				}
			GUI.ObjDraw(RES)
			}
			
			
e3:	
		}

}

	sub do_hex(float n) {
		byte j
		
		float_to_hex(n)
		j = GUI.ObjFind(HEX)
		void string.copy(answer,object.txdata[j])

		
		GUI.ObjDraw(HEX)								

	}
	
	sub do_bin(float n) {
		byte j
		
		
		float_to_bin(n)
		
		j = GUI.ObjFind(BIN)
		void string.copy(banswer,object.txdata[j])
		
		GUI.ObjDraw(BIN)								

	}	

	sub do_oper() {
		;&uword rstr = $0100
		;byte f
		
		;f = GUI.ObjFind(RES)
		
		when OPER  {
		
			ADD -> {
				result = t1 + t2				
			}
			
			MUL -> {
				%asm{{					;check that multiplication does not result in overflow		
					lda p8_t1				
					clc				
					adc p8_t2				
					bcc p8_m1				
					and #$80				
					bne p8_m2				
				}}			
m1:			
				result = t1 * t2															
												
			}	
			
			POW -> {
				result = floats.pow(t1,t2)
			}
			
			TRU -> {
				result = floats.floor(t1)
			}
			
			SUB -> {
				result = t1 - t2
			}
			
			SQT -> {			
				floats.MOVFM(&t1)
				;result = t1 * 1.0		;get t1 into fac1

				floats.SQR()

				floats.MOVMF(&result)
			
				}
				
				
			SIN -> {				
				result = floats.rad(t1)
				result = floats.sin(result)					
			}

			COS -> {				

					result = floats.rad(t1)
					result = floats.cos(result)															
			}
				
			TAN -> {				
				result = floats.rad(t1)
				result = floats.tan(result)					
			}
				
			
			DIV -> {
;				%asm{{			;attempt to detect overflow....doesn't work
;					lda p8_t2
;					sec
;					sbc p8_t1
;					cmp #$ff
;					beq p8_m2
;					}}
				if t2 != 0 {
					result = t1 / t2 }
				else {
m2:					result = t1
					void string.copy("error",rs)
					goto d1
					}
			}
				
		}
	
		
		void floats.FOUT()
		
		void string.copy($0100,rs)	
					
				
		if string.find(rs,46) == 0 {		;not working????
			cat(".",rs) }

			
d1:		

		GUI.ObjDraw(RES)		
		
	}

	sub do_digit(str val, ubyte n) {
		byte f
		float z

		when status {			
			XPO1 -> {
				z = xp
				if string.length(ts1) <= 11 {
					cat(val,ts1) }			
							
				f = GUI.ObjFind(RES)			
				object.txdata[f] = ts1			
				
				xp = do_number(n as float,z)
				
			}
			
			XPO2 -> {
				z = xp
				if string.length(ts2) <= 11 {
					cat(val,ts2) }			
							
				f = GUI.ObjFind(RES)			
				object.txdata[f] = ts2			
				
				xp = do_number(n as float,z)
				
			}			
		
			TERM1 -> {
				z = t1
				if string.length(ts1) == 0 {
					ts1 = val 			
				}			
				else { 
					if string.length(ts1) <= 11 {
						cat(val,ts1) }
					}
				
				t1 = do_number(n as float,z)		
				

				f = GUI.ObjFind(RES)						
				object.txdata[f] = ts1 				
						
			}			
						
									
			TERM2 -> {
				z = t2
				if string.length(ts2) == 0 {
					ts2 = val }
				else { 
					if string.length(ts2) <= 11 {
						cat(val,ts2)
					}
				}
					t2 = do_number(n as float,z)				
							
				f = GUI.ObjFind(RES)						
				object.txdata[f] = ts2 							
			
			}
		}	
	GUI.ObjDraw(RES)
	}

	sub cat(str s1, str s2) {
		ubyte e1
		ubyte e2
		ubyte n
		
		e1 = string.length(s1)
		e2 = string.length(s2)
	

		if s1[0] == 45 {							;-
				for n in e2 downto 0 step -1 {
					s2[n+1] = s2[n] }
				s2[0] = 45
			goto c1
			}
			
		if s1[0] == 43 {							;+
			for n in 1 to e2 {
				s2[n-1] = s2[n] }
		}	
		else {
			s2[e2] = s1[0]
			s2[e2+1] = 0
		}
c1:		
	}
	
;negates the exponent portion of a scientific number
	sub ins(str s1, str s2) {
		ubyte e1
		ubyte e2
		ubyte f
		ubyte n
		
		e1 = string.length(s1)
		e2 = string.length(s2)
		f = string.find(s2,69)

		
		if s1[0] == 45 {							;-
				for n in e2 downto f+1 step -1 {
					s2[n+1] = s2[n] }
				s2[f+1] = 45

			goto i1
			}
			
		if s1[0] == 43 {							;+
			for n in f+2 to e2 {
				s2[n-1] = s2[n] }
		}	

i1:		
	}	
	
	sub do_number(float n, float tot) -> float {
	
		if decimal == false {
			if place == 0 {
				tot = n }
			else {
				tot = tot*10 + n 
			}
			place += 1
		}
		if decimal == true {
			if tot == 0 {
				if place == -1 {
					tot = n / 10 
				}
				else {
					tot = n * floats.pow(10,place as float)
				}
			}
			else {
				tot = tot + n * floats.pow(10,place as float)
			}
			
			place -= 1
		}
		
		return(tot)
		
	}
	
;convert decimal values 0 thru 1 megabyte to hexadecimal	
	sub float_to_hex(float inp) {
	
		float rem
		float res
		float tmp
		ubyte n
		ubyte[20] acc
		ubyte p
		ubyte i
		
		answer[0] = 0
		
		;0 to 1 MB
		if inp < 0 or inp > 1048576 {
		 void string.copy("out of range",answer)
			goto h1 }
		
		if inp <= 15.0 {
		   	answer[0] = num[inp as ubyte]
			answer[1] = 0		
			goto h1
		}
		
		if (inp - floats.floor(inp)) > 0 {

			tmp = inp - floats.floor(inp)  
			inp = inp - tmp }
		
		i = 0
		res = inp/16.0

		repeat {		
			if (res - floats.floor(res)) >= 0 {
				rem = res - floats.floor(res)
				res = res - rem 
			}					
			
			rem = rem * 16.0 
			
			acc[i] = rem as ubyte
						
			i += 1
			
			if res > 0 and res <= 15 {
				acc[i] = res as ubyte
				break
			}
		res = res/16.0	
		}	
	
	p = i
	for n in 0 to i {
			answer[n] = num[acc[p] as ubyte] 
			p -= 1 }
		answer[i+1] = 0		
h1:
	}
	
;convert decimal values 0 thru 256 to binary	
	sub float_to_bin(float inp) {
	
		float rem
		float res
		float tmp
		ubyte n
		ubyte[20] acc
		ubyte p
		ubyte i
		
		banswer[0] = 0
		
		;0 to 256 bytes
		if inp < 0 or inp > 256 {
		 void string.copy("out of range",banswer)
			goto b1 }
		
		if inp <= 2 {
		   	banswer[0] = bnum[inp as ubyte]
			banswer[1] = 0		
			goto b1
		}
		
		if (inp - floats.floor(inp)) > 0 {

			tmp = inp - floats.floor(inp)  
			inp = inp - tmp }
		
		i = 0
		res = inp/2

		repeat {		
			if (res - floats.floor(res)) >= 0 {
				rem = res - floats.floor(res)
				res = res - rem 
			}
							
			rem = rem * 2.0 
			
			acc[i] = rem as ubyte
						
			i += 1
			
			if res > 0 and res <= 1 {
				acc[i] = res as ubyte
				break
			}
		res = res/2.0	
		}
	
	
	p = i
	
	
	for n in 0 to i {
			banswer[n] = bnum[acc[p] as ubyte] 
			p -= 1 }
		banswer[i+1] = 0		
			
b1:
	}	
	
		
;setup the GUI objects
 sub init() {

	;byte res
	
;	gfx2.rect(2,5,316,230,0)

;Buttons
	;decimal digits 0-9
	void GUI.ObjDef(ZER,1,160,205,20,20,iso:"0",CGH,0,1) 
	void GUI.ObjDef(ONE,1,160,180,20,20,iso:"1",CGH,0,1) 
	void GUI.ObjDef(TWO,1,185,180,20,20,iso:"2",CGH,0,1) 
	void GUI.ObjDef(THR,1,210,180,20,20,iso:"3",CGH,0,1) 
	void GUI.ObjDef(FOR,1,160,155,20,20,iso:"4",CGH,0,1) 	
	void GUI.ObjDef(FIV,1,185,155,20,20,iso:"5",CGH,0,1) 
	void GUI.ObjDef(SIX,1,210,155,20,20,iso:"6",CGH,0,1) 	
	void GUI.ObjDef(SEV,1,160,130,20,20,iso:"7",CGH,0,1) 
	void GUI.ObjDef(ATE,1,185,130,20,20,iso:"8",CGH,0,1) 
	void GUI.ObjDef(NIN,1,210,130,20,20,iso:"9",CGH,0,1) 	
		
	;Pi
	void GUI.ObjDef(PIE,1,235,130,20,20,PIC,CGH,0,1)	
		
	;decimal point	
	void GUI.ObjDef(DOT,1,210,205,20,20,iso:".",CGH,0,1) 
	

	;math operators
	void GUI.ObjDef(EQU,1,260,205,20,20,iso:"=",CGH,0,1)
	void GUI.ObjDef(ADD,1,235,180,20,20,iso:"+",CGH,0,1) 
	void GUI.ObjDef(SUB,1,260,180,20,20,iso:"-",CGH,0,1) 	
	void GUI.ObjDef(MUL,1,235,155,20,20,iso:"x",CGH,0,1) 
	void GUI.ObjDef(DIV,1,260,155,20,20,DVS,CGH,0,1) 		
	void GUI.ObjDef(SQT,1,160,105,20,20,RUT,CGH,0,1) 		
	void GUI.ObjDef(SQR,1,185,105,20,20,QRS,CGH,0,1)	
	void GUI.ObjDef(POW,1,210,105,20,20,PWW,CGH,0,1) 		
	void GUI.ObjDef(SIN,1,125,105,30,20,iso:"sin",CGH,0,1)
	void GUI.ObjDef(COS,1,125,130,30,20,iso:"cos",CGH,0,1)
	void GUI.ObjDef(TAN,1,125,155,30,20,iso:"tan",CGH,0,1)
	void GUI.ObjDef(TRU,1,235,105,50,20,iso:"Trunc",CGH,0,1)
		
		
	;clear all	
	void GUI.ObjDef(CLR,1,260,130,20,20,iso:"C",52,0,1) 
	
	;clear memory
	void GUI.ObjDef(MCL,1,285,130,25,20,iso:"MC",52,0,1)
	void GUI.ObjDef(MMR,1,285,155,25,20,iso:"MR",CGH,0,1)
	void GUI.ObjDef(MMS,1,285,180,25,20,iso:"MS",CGH,0,1)
	void GUI.ObjDef(MMP,1,285,205,25,20,iso:"M+",CGH,0,1)
	
	
	void GUI.ObjDef(EXP,1,235,205,20,20,XPE,CGH,0,1)
	
	void GUI.ObjDef(NEG,1,185,205,20,20,NGS,CGH,0,1) 
	
;text boxes
	void GUI.ObjDef(RES,5,160,14,150,16,iso:"0.",1,0,34) 
	void GUI.ObjDef(HEX,5,160,36,150,16,iso:"$0",1,0,34) 
	void GUI.ObjDef(BIN,5,160,58,150,16,iso:"%0",1,0,34) 
	
	
	 
	void GUI.ObjDef(HLP,5,8,12,138,16,iso:"A GUI Calculator",CGH,0,0)
	void GUI.ObjDef(HIN,5,130,36,25,16,iso:"HEX",CGH,0,0)
	void GUI.ObjDef(BNN,5,130,58,25,16,iso:"BIN",CGH,0,0)
	void GUI.ObjDef(IND,5,145,17,12,10,iso:"",CGH,0,16)

;frame - must be last object defined, since all other objects are inside it! 	
	void GUI.ObjDef(TTL,9,5,7,310,228,iso:"Calca",CGH,0,4) 	


;set the properties of the object, visibility, enabled etc...
	void GUI.ObjPSet(ZER,%00001100)
	void GUI.ObjPSet(ONE,%00001100)
	void GUI.ObjPSet(TWO,%00001100)
	void GUI.ObjPSet(THR,%00001100)
	void GUI.ObjPSet(FOR,%00001100)
	void GUI.ObjPSet(FIV,%00001100)	
	void GUI.ObjPSet(SIX,%00001100)
	void GUI.ObjPSet(SEV,%00001100)
	void GUI.ObjPSet(ATE,%00001100)
	void GUI.ObjPSet(NIN,%00001100)		
	
	void GUI.ObjPSet(PIE,%00001100)	
	void GUI.ObjPSet(SIN,%00001100)
	void GUI.ObjPSet(COS,%00001100)
	void GUI.ObjPSet(TAN,%00001100)
	
	void GUI.ObjPSet(DOT,%00001100)		
	void GUI.ObjPSet(EXP,%00001100)
	void GUI.ObjPSet(EQU,%00001100)
	void GUI.ObjPSet(ADD,%00001100)		
	void GUI.ObjPSet(SUB,%00001100)	
	void GUI.ObjPSet(MUL,%00001100)		
	void GUI.ObjPSet(DIV,%00001100)	
	void GUI.ObjPSet(CLR,%00001100)
	void GUI.ObjPSet(EXP,%00001100)
	void GUI.ObjPSet(SQT,%00001100)		
	void GUI.ObjPSet(SQR,%00001100)		
	void GUI.ObjPSet(POW,%00001100)		
	void GUI.ObjPSet(MCL,%00001100)
	void GUI.ObjPSet(MMS,%00001100)
	void GUI.ObjPSet(MMP,%00001100)
	void GUI.ObjPSet(MMR,%00001100)
	void GUI.ObjPSet(TRU,%00001100)
			
	void GUI.ObjPSet(NEG,%00101100)		
		
	void GUI.ObjPSet(RES,%00101000)
	void GUI.ObjPSet(HEX,%00101000)	
	void GUI.ObjPSet(BIN,%00101000)	
	void GUI.ObjPSet(HIN,%00101000)
	void GUI.ObjPSet(BNN,%00101000)
	
	
	void GUI.ObjPSet(HLP,%00101000)	
	void GUI.ObjPSet(IND,%00001000)
	void GUI.ObjPSet(TTL,%00101000)	



;nothing appears on the screen until you draw the object
;this allows multiple updates at once and you control when it happens
	GUI.ObjDraw(TTL)
	GUI.ObjDraw(ZER)
	GUI.ObjDraw(ONE)
	GUI.ObjDraw(TWO)
	GUI.ObjDraw(THR)	
	GUI.ObjDraw(FOR)
	GUI.ObjDraw(FIV)	
	GUI.ObjDraw(SIX)
	GUI.ObjDraw(SEV)	
	GUI.ObjDraw(ATE)
	GUI.ObjDraw(NIN)		
		
	GUI.ObjDraw(PIE)
	GUI.ObjDraw(SIN)
	GUI.ObjDraw(COS)
	GUI.ObjDraw(TAN)

		
	GUI.ObjDraw(DOT)
	GUI.ObjDraw(EXP)
	GUI.ObjDraw(EQU)
	GUI.ObjDraw(ADD)
	GUI.ObjDraw(SUB)	
	GUI.ObjDraw(ADD)
	GUI.ObjDraw(MUL)	
	GUI.ObjDraw(DIV)
	GUI.ObjDraw(SQT)	
	GUI.ObjDraw(SQR)
	GUI.ObjDraw(POW)
	GUI.ObjDraw(TRU)
	
	GUI.ObjDraw(CLR)
	GUI.ObjDraw(MCL)
	GUI.ObjDraw(MMP)
	GUI.ObjDraw(MMS)
	GUI.ObjDraw(MMR)
	GUI.ObjDraw(NEG)
	GUI.ObjDraw(RES)
	GUI.ObjDraw(HEX)	
	GUI.ObjDraw(BIN)		
	GUI.ObjDraw(IND)
;	ObjDraw(TTL)
	GUI.ObjDraw(HLP)
	GUI.ObjDraw(HIN)
	GUI.ObjDraw(BNN)	

}






}	;end of main

GUI {

	uword Xt
	uword Yt
	ubyte CGH = main.CGH
	ubyte CGY = main.CGY

;fetches object value, which depends on object type. Might be whether or not object is selected/checked or actual numerical value.	
	sub ObjVGet(ubyte id) -> uword {
		byte n
		
		n = ObjFind(id)
		if n > -1 {
			when object.type[n] { 
				2,3 -> {
					return (object.prop[n] & 32) 
					}
				4,8 -> {
					return (object.value[n])
					}
			}
		}	
	}

;waits for the stylus to be lifted off the panel	
	sub WaitNoTouch() {
		do {
			main.btn = cx16.mouse_get(&main.mx as ubyte)
		} until main.btn & %00000001 == 0 		
	}
	
;populates x,y coordinates if left mouse button pressed (a touch) and returns true, else returns false for no touch 	
	sub GetTouch() -> bool {
		main.btn = cx16.mouse_get(&main.mx as ubyte)
		
		if main.btn & %00000001 != 0 {
			Xt = main.mx
			Yt = main.my
			return(true)
		}
		else {
			return(false)
		}	
	}
	
;populates x,y coordinates if right mouse button pressed (a touch) and returns true, else returns false for no touch 	
	sub GetAltTouch() -> bool {
		main.btn = cx16.mouse_get(&main.mx as ubyte)
		
		if main.btn & %00000010 != 0 {
			Xt = main.mx
			Yt = main.my
			return(true)
		}
		else {
			return(false)
		}	
	}	
	
;Main touch processor. Regular calls here will get the numeric name of
;screen objects that are touched, return -1 if no object is touched
;animates and manages status etc
	sub ProcessTouch() -> byte {
		byte Pt
		ubyte n
		ubyte m
		uword w
		ubyte q
		
		Pt = -1
			for n in 0 to main.P-1 as ubyte {
				if Xt >= object.xpos[n] { ;and Xt <= object.width[n] {
					if Yt >= object.ypos[n] { ;and Yt <= object.height[n] {
						if Xt <= object.width[n] {
							if Yt <= object.height[n] {
								if object.prop[n] & %00001000 {
									if object.prop[n] & %00000100 {
										Pt = object.id[n] as byte
										object.prop[n] = object.prop[n] | %00010000   	;touched
										when object.type[n] {								;click animation
											1 -> {											;button
												object.opt3[n] = 2
												ObjDraw(object.id[n])
											}												;end case 1
											
											2 -> {											;radio button
												for m in 0 to main.P as ubyte {
													if object.id[n] != object.id[m] {
														if object.type[m] == 2 {
															if object.opt1[m] == object.opt1[n] {
																object.prop[m] = object.prop[m] & %11011111
																ObjDraw(object.id[m])
															}
														}
													}
												}
												object.prop[n] = object.prop[n] | %00100000
												ObjDraw(object.id[n])												
											}												;end case 2

											3 -> {											;checkbox
												if object.prop[n] & %00100000 {
													object.prop[n] = object.prop[n] & %11011111
												}
												else {
													object.prop[n] = object.prop[n] | %00100000
												}
												ObjDraw(object.id[n])
											}												;end case 3
											
											4 -> {											;slider												
												do {
														w = object.width[n] - object.xpos[n]
														Xt = (floats.clampf(Xt as float - object.xpos[n] as float,0.0,w as float)) as uword
														w = (Xt as float/(w as float/(object.opt2[n] - object.opt1[n]) as float)) as uword
														ObjVSet(object.id[n],w)
														if object.opt3[n] {
															ubyte t
															conv.str_uw(w)
															t = ObjFind(object.opt3[n] as ubyte) as ubyte
															object.txdata[t] = conv.string_out
															ObjDraw(object.opt3[n] as ubyte)
														}
													ObjDraw(object.id[n])	
												} until GetTouch() == false	;left button released
											}												;end case 4												
										}													;end case(when structure)											
									
ra:										WaitNoTouch()										;make sure button is released
										when object.type[n] {									;release animation
											1 -> {
												object.opt3[n] = 1												
												ObjDraw(object.id[n])
											}
										} ;end when
										break	;to exit for loop when object found
									}
								}
							}
						}
					}
				}
			}	;end for
			return(Pt)
		}	;end processtouch()	
	

;set values of gadget by its numeric name
 sub ObjVSet(ubyte id, uword v) {
		byte n
		float lo
		float hi
		
		
		n = ObjFind(id)
		
		if n > -1 {
			when object.type[n] { 
				4,8 -> {
					lo = object.opt1[n] as float
					hi = object.opt2[n] as float
 					object.value[n] = floats.clampf(v as float,lo,hi) as uword
				}
			}
		} 
 }

 sub Frame(byte n, ubyte os) {
 	ubyte c1
 	ubyte c2
 	
 		when (object.opt3[n] & $f) {
			0 -> {							;none
				c1 = CGH
				c2 = CGH
				}
				
			1 -> {							;button style raised
				c1 = 1
				c2 = 0
				}
				
			2 -> {							;button style depressed
				c1 = 0
				c2 = 1
				}
				
			3 -> {							;white
				c1 = 1
				c2 = 1
				}
				
			4 -> {							;black
				c1 = 0
				c2 = 0
				}
				
			5 -> {							;grey (disabled)
				c1 = CGY
				c2 = CGY
				}
			}
		gfx2.line(object.xpos[n]-os,object.ypos[n]-os,object.xpos[n]-os,object.height[n]+os,c1)
		gfx2.line(object.xpos[n]-os,object.ypos[n]-os,object.width[n]+os,object.ypos[n]-os,c1)
		gfx2.line(object.width[n]+os,object.ypos[n]-os,object.width[n]+os,object.height[n]+os,c2)
		gfx2.line(object.xpos[n]-os,object.height[n]+os,object.width[n]+os,object.height[n]+os,c2)
			
 }

 sub ObjCleanup(byte n, ubyte f) {

		if f or (object.prop[n] & 64) {
			gfx2.fillrect(object.xpos[n]-1,object.ypos[n]-1,object.width[n]+1,object.height[n]+1,CGH)
		}
		object.prop[n] = object.prop[n] & $bf					;clear cleanup flag

 }

 sub ObjDraw( ubyte id) {
 		byte o
		byte n
		byte m
		uword x
		uword y
		uword w
		uword h
		uword z
		uword xx
		uword yy
	 	float nn
		uword j
		;str a
		ubyte tc				;text color
		
		n = ObjFind(id)
		if n > -1 {
			ObjCleanup(n,0)
			x = object.xpos[n]
			y = object.ypos[n]
			w = object.width[n] - x
			h = object.height[n] - y
			
			when object.type[n] {
				1 -> {															;button
					if object.prop[n] & 8 {										;visible
						gfx2.fillrect(x,y,w,h,object.opt1[n] as ubyte)
						Frame(n,0)
						if object.prop[n] & 4 {									;enabled
							z = object.opt2[n] 
						}
						else {													;ghosted
							z = 24
						}
						gfx2.text(w/2+x-8*string.length(object.txdata[n])/2,h/2+y-8/2,z as ubyte,object.txdata[n])
					}
					else {														;erase
						ObjCleanup(n,1)
						}
				}
				
				2 -> {																;radio
					if object.prop[n] & %00001000 {									;visible
						x += 5
						y += 5
						tc = 0
						if object.prop[n] & %00000100 {								;Enabled
							if object.prop[n] & %00100000 {							;set
								gfx2.disc(x,y,5,26)
								gfx2.disc(x,y,4,1)
								gfx2.disc(x,y,3,object.opt2[n] as ubyte)
		
								nn = 45.0
								while nn <= 120.0 {
									xx = (x as float + 0.5 + 5.0*floats.sin(nn)) as uword
									yy = (y as float + 0.5 + 5.0*floats.cos(nn)) as uword
									gfx2.plot(xx,yy,0)
									nn += 6.0
								}

							}
							else {														;unset
									gfx2.disc(x,y,5,26)
									gfx2.disc(x,y,4,1)
									gfx2.disc(x,y,3,29)
		
									nn = 45.0
									while nn <= 120.0 {
										xx = (x as float + 0.5 + 5.0*floats.sin(nn)) as uword
										yy = (y as float + 0.5 + 5.0*floats.cos(nn)) as uword
										gfx2.plot(xx,yy,0)
										nn += 6.0
									}
								}							
						}						
						else {																;disabled
							gfx2.disc(x,y,5,26)
							gfx2.disc(x,y,4,29)
							gfx2.disc(x,y,3,29)
							tc = 24
						}								
						gfx2.text(x+15,y+(h/2)-7,tc,object.txdata[n])
						
					}

				}
				
	
			
				3 -> {																		;checkbox 
					if object.prop[n] & %00001000 {										;visible		
						if object.prop[n] & %00000100 {									;Enabled
							tc = 1 }
						else {																;ghosted
							tc = 29
						}
						gfx2.fillrect(x,y,11,11,tc)
						gfx2.line(x,y,x,y+11,0)
						gfx2.line(x,y,x+11,y,0)
						gfx2.line(x+11,y,x+11,y+11,1)
						gfx2.line(x+11,y+11,x,y+11,1)
						
						if object.prop[n] & %00000100 {									;Enabled
							tc = 0 }
						else {																;ghosted
							tc = 24
						}
						if object.prop[n] & %00100000 {									;checked
;							gfx2.line(x+2,y+2,x+9,y+9,tc)
;							gfx2.line(x+2,y+9,x+9,y+2,tc)
							gfx2.line(x+2,y+6,x+5,y+9,tc)
							gfx2.line(x+5,y+9,x+9,y+2,tc)

							}						
						gfx2.text(x+15,y+(h/2)-2,tc,object.txdata[n])					
					}				
					else {																	;erase
						ObjCleanup(n,1) }
					}
					
				4 -> {																		;slider
					if object.prop[n] & %00001000	{										;visible
						gfx2.fillrect(x-5,y,w+11,h+1,CGH)
						y=y+(h/2)
						gfx2.line(x,y-2,x+w,y-2,0)
						gfx2.line(x,y-2,x,y+2,0)
						gfx2.line(x,y+2,x+w,y+2,1)
						gfx2.line(x+w,y+2,x+w,y-2,1)
						x = x as float + ((w as float/(object.opt2[n]-object.opt1[n]) as float)*object.value[n] as float) as uword
						y = object.ypos[n]
						;pointer
						if object.prop[n] & %00000100 {												;Enabled
							gfx2.fillrect(x-5,y,11,h,CGH)
							
							gfx2.line(x+5,y+3,x+5,y+h,0)
							gfx2.line(x+5,y+h,x-5,y+h,0)
							
							gfx2.line(x,y,x+5,y+3,1)
							
							gfx2.line(x-5,y+h,x-5,y+3,1)
							gfx2.line(x-5,y+3,x,y,1)
						}
						else {																			;disabled
							gfx2.fillrect(x-5,y,11,h,CGY)
							gfx2.line(x+5,y+3,x+5,y+h,CGH)
							gfx2.line(x+5,y+h,x-5,y+h,CGH)
							gfx2.line(x,y,x+5,y+3,CGH)
							gfx2.line(x-5,y+h,x-5,y+3,CGH)
							gfx2.line(x-5,y+3,x,y,CGH)
						}
					}
					else {																				;erase
						ObjCleanup(n,1)
						}
					}
									
					
				5 -> {																					;textbox
					if object.prop[n] & %00001000	{													;visible
						gfx2.fillrect(x,y,w+1,h+1,object.opt1[n] as ubyte)
						Frame(n,1)
						when object.opt3[n] >> 4 {
							0 -> {																		;Left justified
								gfx2.text(x,h/2+y-8/2+1,object.opt2[n] as ubyte,object.txdata[n])
								}
							1 -> {																		;Center justified
								gfx2.text(w/2+x-8*string.length(object.txdata[n])/2,h/2+y-8/2+1,object.opt2[n] as ubyte,object.txdata[n])
								}
							2 -> {																		;Right justified
								gfx2.text(object.width[n]-(8*string.length(object.txdata[n])),h/2+y-8/2+1,object.opt2[n] as ubyte,object.txdata[n])
								}
							}
						}
					else {																				;erase
						ObjCleanup(n,1)
					}
				}					
								
				8 -> {																					;progress bar
					if object.prop[n] & %00001000 {														;visible
						Frame(n,1)
						
						if object.opt3[n] & %00010000 {													;vert
							z = (h/(object.opt2[n]-object.opt1[n]))*object.value[n]
							j = object.value[n]
							gfx2.fillrect(x,y,w,h-z,CGY)
							tc = (object.opt3[n] >> 8) as ubyte
							gfx2.fillrect(x,(y+h)-z,w,z,tc)
							goto p1
						}																				
						if object.opt3[n] & %00010000 == false {																			;horz
							z = (w/(object.opt2[n]-object.opt1[n]))*object.value[n]						
							gfx2.fillrect(x+z,y,w-z,h,CGY)
							tc = (object.opt3[n] >> 8) as ubyte
							gfx2.fillrect(x,y,z,h,tc)
							}
						}
					else {																				;erase
						ObjCleanup(n,1)
					}
p1:
				}								
					
					
				9 -> {																					;frame
					Frame(n,0)
					gfx2.line(x+5,y,x+5+string.length(object.txdata[n])*8,y,CGH)
					gfx2.text(x+5,y-8/2,0,object.txdata[n])
					}					
				}
				
			}
			
		;} 
 	
 
 }
  

;Sets the properties of an object
;Returns -1 if not found
	sub ObjPSet(ubyte id, ubyte b) -> byte {					;id,enabled,visible,touched,Set,cleanup
		byte n
		
		n = ObjFind(id)
		if n > -1 {
			object.prop[n] = b as byte
			return(1 as byte)								;just a non negative value
		}
		return(-1)									;object not found
	}
	
;Finds the internal reference for an object from its numeric name
;Returns the object number or -1 if not found
	sub ObjFind(ubyte id) -> byte {
		ubyte n
		
		for n in 0 to main.P-1 as ubyte {
			if object.id[n] == id {
				return(n as byte) }						;object exists
		}
		return(-1)
	}	
	
;defines an object but doesn't draw anything. returns 0 if all OK.
;res=GUIObjDef(21,1,10,10,80,30,"Save",&hffggbb,,1)
	sub ObjDef(ubyte id, ubyte tp, uword x, uword y, uword w, uword h, str tx, uword g, uword c, uword d) -> ubyte {
		byte n
		
		if main.P > main.Objs {				;too many objects
			return(1)							
		}							
		n = ObjFind(id)
		
		if n == -1 {
			n = main.P
			main.P += 1					;define or redefine. P always points to the next free "slot"
		}
		
		object.id[n as ubyte] = id
		object.type[n as ubyte] = tp
		object.prop[n as ubyte] = 0
		object.xpos[n as ubyte] = x
		object.ypos[n as ubyte] = y
		object.opt1[n as ubyte] = g			
		object.opt2[n as ubyte] = c
		object.opt3[n as ubyte] = d
		object.value[n as ubyte] = g
		object.txdata[n as ubyte] = tx
	
		
		when tp {
			1,5,8,9 -> {									;button,textbox,progress bar,frame
				object.width[n] = x+w
				object.height[n] = y+h
				if object.opt1[n] + object.opt2[n] == 0 { 
					object.opt1[n] = CGY
					}
			}
			
			2,3	-> {										;Radio,checkbox
				object.width[n] = x + 15 + 8 * string.length(object.txdata[n])
				object.height[n] = y + 8
				}
				
			4 -> {											;Slider
				object.ypos[n] = y - 4
				object.width[n] = w ;x + w
				object.height[n] = y + 14
				}
		}
	return(0)
	}		
	
}

; data structure for GUI objects
object {
	ubyte[main.Objs] id 			;object unique id
	ubyte[main.Objs] type 			;type of gadget
	ubyte[main.Objs] prop 			;properties bits
	uword[main.Objs] xpos 			
	uword[main.Objs] ypos 
	uword[main.Objs] width 			
	uword[main.Objs] height 		
	uword[main.Objs] opt1 			;optional parameters - min., max. , etc.
	uword[main.Objs] opt2 			;optional parameters
	uword[main.Objs] opt3 			;border type, etc.
	uword[main.Objs] value 			;optional value of object
	; 51 strings
	str[] txdata = ["??????????????????????????????","??????????????????????????????","??????????????????????????????",
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",	
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",	
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",	
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",	
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",	
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",	
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",	
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",	
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",	
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",	
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",	
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",						
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",							
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",							
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",							
						"??????????????????????????????","??????????????????????????????","??????????????????????????????",																			
						"??????????????????????????????","??????????????????????????????","??????????????????????????????"]	
	}

	
	
	